/*
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
/*
 * Warning! This file is generated. Modify at your own risk.
 */

package com.google.api.services.shopping.model;

import com.google.api.client.json.GenericJson;

/**
 * Model definition for Products.
 *
 * <p> This is the Java data model class that specifies how to parse/serialize into the JSON that is
 * transmitted over HTTP when working with the Search API For Shopping. For a detailed explanation
 * see:
 * <a href="http://code.google.com/p/google-api-java-client/wiki/Json">http://code.google.com/p/google-api-java-client/wiki/Json</a>
 * </p>
 *
 * <p>
 * Upgrade warning: starting with version 1.12 {@code getResponseHeaders()} is removed, instead use
 * {@link com.google.api.client.http.json.JsonHttpRequest#getLastResponseHeaders()}
 * </p>
 *
 * @author Google, Inc.
 */
@SuppressWarnings("javadoc")
public final class Products extends GenericJson {

  /**
   * List of categories.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.List<ShoppingModelCategoryJsonV1> categories;

  static {
    // hack to force ProGuard to consider ShoppingModelCategoryJsonV1 used, since otherwise it would be stripped out
    // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
    com.google.api.client.util.Data.nullOf(ShoppingModelCategoryJsonV1.class);
  }

  /**
   * Recommendations for category.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.List<ShoppingModelRecommendationsJsonV1> categoryRecommendations;

  static {
    // hack to force ProGuard to consider ShoppingModelRecommendationsJsonV1 used, since otherwise it would be stripped out
    // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
    com.google.api.client.util.Data.nullOf(ShoppingModelRecommendationsJsonV1.class);
  }

  /**
   * Current item count.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Integer currentItemCount;

  /**
   * Google internal.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private ShoppingModelDebugJsonV1 debug;

  /**
   * Etag of feed.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String etag;

  /**
   * List of extras.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private ShoppingModelExtrasJsonV1 extras;

  /**
   * List of facets.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.List<Facets> facets;

  static {
    // hack to force ProGuard to consider Facets used, since otherwise it would be stripped out
    // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
    com.google.api.client.util.Data.nullOf(Facets.class);
  }

  /**
   * Id of feed.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String id;

  /**
   * List of returned products.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.List<Product> items;

  static {
    // hack to force ProGuard to consider Product used, since otherwise it would be stripped out
    // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
    com.google.api.client.util.Data.nullOf(Product.class);
  }

  /**
   * Number of items per page of results.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Integer itemsPerPage;

  /**
   * The fixed string "shopping#products". The kind of feed returned.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String kind;

  /**
   * Next link of feed.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String nextLink;

  /**
   * Previous link of feed.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String previousLink;

  /**
   * List of promotions.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.List<Promotions> promotions;

  static {
    // hack to force ProGuard to consider Promotions used, since otherwise it would be stripped out
    // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
    com.google.api.client.util.Data.nullOf(Promotions.class);
  }

  /**
   * Redirects.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.List<String> redirects;

  /**
   * Related queries.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.List<String> relatedQueries;

  /**
   * Unique identifier for this request.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String requestId;

  /**
   * Self link of feed.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String selfLink;

  /**
   * Spelling.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Spelling spelling;

  /**
   * 1-based index of the first item in the search results.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Integer startIndex;

  /**
   * List of returned stores.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.List<Stores> stores;

  static {
    // hack to force ProGuard to consider Stores used, since otherwise it would be stripped out
    // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
    com.google.api.client.util.Data.nullOf(Stores.class);
  }

  /**
   * Total number of search results.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Integer totalItems;

  /**
   * List of categories.
   * The value returned may be {@code null}.
   */
  public java.util.List<ShoppingModelCategoryJsonV1> getCategories() {
    return categories;
  }

  /**
   * List of categories.
   * The value set may be {@code null}.
   */
  public Products setCategories(java.util.List<ShoppingModelCategoryJsonV1> categories) {
    this.categories = categories;
    return this;
  }

  /**
   * Recommendations for category.
   * The value returned may be {@code null}.
   */
  public java.util.List<ShoppingModelRecommendationsJsonV1> getCategoryRecommendations() {
    return categoryRecommendations;
  }

  /**
   * Recommendations for category.
   * The value set may be {@code null}.
   */
  public Products setCategoryRecommendations(java.util.List<ShoppingModelRecommendationsJsonV1> categoryRecommendations) {
    this.categoryRecommendations = categoryRecommendations;
    return this;
  }

  /**
   * Current item count.
   * The value returned may be {@code null}.
   */
  public Integer getCurrentItemCount() {
    return currentItemCount;
  }

  /**
   * Current item count.
   * The value set may be {@code null}.
   */
  public Products setCurrentItemCount(Integer currentItemCount) {
    this.currentItemCount = currentItemCount;
    return this;
  }

  /**
   * Google internal.
   * The value returned may be {@code null}.
   */
  public ShoppingModelDebugJsonV1 getDebug() {
    return debug;
  }

  /**
   * Google internal.
   * The value set may be {@code null}.
   */
  public Products setDebug(ShoppingModelDebugJsonV1 debug) {
    this.debug = debug;
    return this;
  }

  /**
   * Etag of feed.
   * The value returned may be {@code null}.
   */
  public String getEtag() {
    return etag;
  }

  /**
   * Etag of feed.
   * The value set may be {@code null}.
   */
  public Products setEtag(String etag) {
    this.etag = etag;
    return this;
  }

  /**
   * List of extras.
   * The value returned may be {@code null}.
   */
  public ShoppingModelExtrasJsonV1 getExtras() {
    return extras;
  }

  /**
   * List of extras.
   * The value set may be {@code null}.
   */
  public Products setExtras(ShoppingModelExtrasJsonV1 extras) {
    this.extras = extras;
    return this;
  }

  /**
   * List of facets.
   * The value returned may be {@code null}.
   */
  public java.util.List<Facets> getFacets() {
    return facets;
  }

  /**
   * List of facets.
   * The value set may be {@code null}.
   */
  public Products setFacets(java.util.List<Facets> facets) {
    this.facets = facets;
    return this;
  }

  /**
   * Id of feed.
   * The value returned may be {@code null}.
   */
  public String getId() {
    return id;
  }

  /**
   * Id of feed.
   * The value set may be {@code null}.
   */
  public Products setId(String id) {
    this.id = id;
    return this;
  }

  /**
   * List of returned products.
   * The value returned may be {@code null}.
   */
  public java.util.List<Product> getItems() {
    return items;
  }

  /**
   * List of returned products.
   * The value set may be {@code null}.
   */
  public Products setItems(java.util.List<Product> items) {
    this.items = items;
    return this;
  }

  /**
   * Number of items per page of results.
   * The value returned may be {@code null}.
   */
  public Integer getItemsPerPage() {
    return itemsPerPage;
  }

  /**
   * Number of items per page of results.
   * The value set may be {@code null}.
   */
  public Products setItemsPerPage(Integer itemsPerPage) {
    this.itemsPerPage = itemsPerPage;
    return this;
  }

  /**
   * The fixed string "shopping#products". The kind of feed returned.
   * The value returned may be {@code null}.
   */
  public String getKind() {
    return kind;
  }

  /**
   * The fixed string "shopping#products". The kind of feed returned.
   * The value set may be {@code null}.
   */
  public Products setKind(String kind) {
    this.kind = kind;
    return this;
  }

  /**
   * Next link of feed.
   * The value returned may be {@code null}.
   */
  public String getNextLink() {
    return nextLink;
  }

  /**
   * Next link of feed.
   * The value set may be {@code null}.
   */
  public Products setNextLink(String nextLink) {
    this.nextLink = nextLink;
    return this;
  }

  /**
   * Previous link of feed.
   * The value returned may be {@code null}.
   */
  public String getPreviousLink() {
    return previousLink;
  }

  /**
   * Previous link of feed.
   * The value set may be {@code null}.
   */
  public Products setPreviousLink(String previousLink) {
    this.previousLink = previousLink;
    return this;
  }

  /**
   * List of promotions.
   * The value returned may be {@code null}.
   */
  public java.util.List<Promotions> getPromotions() {
    return promotions;
  }

  /**
   * List of promotions.
   * The value set may be {@code null}.
   */
  public Products setPromotions(java.util.List<Promotions> promotions) {
    this.promotions = promotions;
    return this;
  }

  /**
   * Redirects.
   * The value returned may be {@code null}.
   */
  public java.util.List<String> getRedirects() {
    return redirects;
  }

  /**
   * Redirects.
   * The value set may be {@code null}.
   */
  public Products setRedirects(java.util.List<String> redirects) {
    this.redirects = redirects;
    return this;
  }

  /**
   * Related queries.
   * The value returned may be {@code null}.
   */
  public java.util.List<String> getRelatedQueries() {
    return relatedQueries;
  }

  /**
   * Related queries.
   * The value set may be {@code null}.
   */
  public Products setRelatedQueries(java.util.List<String> relatedQueries) {
    this.relatedQueries = relatedQueries;
    return this;
  }

  /**
   * Unique identifier for this request.
   * The value returned may be {@code null}.
   */
  public String getRequestId() {
    return requestId;
  }

  /**
   * Unique identifier for this request.
   * The value set may be {@code null}.
   */
  public Products setRequestId(String requestId) {
    this.requestId = requestId;
    return this;
  }

  /**
   * Self link of feed.
   * The value returned may be {@code null}.
   */
  public String getSelfLink() {
    return selfLink;
  }

  /**
   * Self link of feed.
   * The value set may be {@code null}.
   */
  public Products setSelfLink(String selfLink) {
    this.selfLink = selfLink;
    return this;
  }

  /**
   * Spelling.
   * The value returned may be {@code null}.
   */
  public Spelling getSpelling() {
    return spelling;
  }

  /**
   * Spelling.
   * The value set may be {@code null}.
   */
  public Products setSpelling(Spelling spelling) {
    this.spelling = spelling;
    return this;
  }

  /**
   * 1-based index of the first item in the search results.
   * The value returned may be {@code null}.
   */
  public Integer getStartIndex() {
    return startIndex;
  }

  /**
   * 1-based index of the first item in the search results.
   * The value set may be {@code null}.
   */
  public Products setStartIndex(Integer startIndex) {
    this.startIndex = startIndex;
    return this;
  }

  /**
   * List of returned stores.
   * The value returned may be {@code null}.
   */
  public java.util.List<Stores> getStores() {
    return stores;
  }

  /**
   * List of returned stores.
   * The value set may be {@code null}.
   */
  public Products setStores(java.util.List<Stores> stores) {
    this.stores = stores;
    return this;
  }

  /**
   * Total number of search results.
   * The value returned may be {@code null}.
   */
  public Integer getTotalItems() {
    return totalItems;
  }

  /**
   * Total number of search results.
   * The value set may be {@code null}.
   */
  public Products setTotalItems(Integer totalItems) {
    this.totalItems = totalItems;
    return this;
  }

  /**
   * Model definition for ProductsFacets.
   */
  public static final class Facets extends GenericJson {

    /**
     * List of Buckets within facet.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<Buckets> buckets;

    static {
      // hack to force ProGuard to consider Buckets used, since otherwise it would be stripped out
      // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
      com.google.api.client.util.Data.nullOf(Buckets.class);
    }

    /**
     * Number of products matching the query that have a value for the facet's property or attribute.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private Integer count;

    /**
     * Display name of facet.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String displayName;

    /**
     * Name of the facet's attribute (omitted for property facets).
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String name;

    /**
     * Property of facet (omitted for attribute facets).
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String property;

    /**
     * Type of facet's attribute (omitted for property facets, one of: text, bool, int, float).
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String type;

    /**
     * Unit of the facet's property or attribute (omitted if the facet's property or attribute has no
     * unit).
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String unit;

    /**
     * List of Buckets within facet.
     * The value returned may be {@code null}.
     */
    public java.util.List<Buckets> getBuckets() {
      return buckets;
    }

    /**
     * List of Buckets within facet.
     * The value set may be {@code null}.
     */
    public Facets setBuckets(java.util.List<Buckets> buckets) {
      this.buckets = buckets;
      return this;
    }

    /**
     * Number of products matching the query that have a value for the facet's property or attribute.
     * The value returned may be {@code null}.
     */
    public Integer getCount() {
      return count;
    }

    /**
     * Number of products matching the query that have a value for the facet's property or attribute.
     * The value set may be {@code null}.
     */
    public Facets setCount(Integer count) {
      this.count = count;
      return this;
    }

    /**
     * Display name of facet.
     * The value returned may be {@code null}.
     */
    public String getDisplayName() {
      return displayName;
    }

    /**
     * Display name of facet.
     * The value set may be {@code null}.
     */
    public Facets setDisplayName(String displayName) {
      this.displayName = displayName;
      return this;
    }

    /**
     * Name of the facet's attribute (omitted for property facets).
     * The value returned may be {@code null}.
     */
    public String getName() {
      return name;
    }

    /**
     * Name of the facet's attribute (omitted for property facets).
     * The value set may be {@code null}.
     */
    public Facets setName(String name) {
      this.name = name;
      return this;
    }

    /**
     * Property of facet (omitted for attribute facets).
     * The value returned may be {@code null}.
     */
    public String getProperty() {
      return property;
    }

    /**
     * Property of facet (omitted for attribute facets).
     * The value set may be {@code null}.
     */
    public Facets setProperty(String property) {
      this.property = property;
      return this;
    }

    /**
     * Type of facet's attribute (omitted for property facets, one of: text, bool, int, float).
     * The value returned may be {@code null}.
     */
    public String getType() {
      return type;
    }

    /**
     * Type of facet's attribute (omitted for property facets, one of: text, bool, int, float).
     * The value set may be {@code null}.
     */
    public Facets setType(String type) {
      this.type = type;
      return this;
    }

    /**
     * Unit of the facet's property or attribute (omitted if the facet's property or attribute has no
     * unit).
     * The value returned may be {@code null}.
     */
    public String getUnit() {
      return unit;
    }

    /**
     * Unit of the facet's property or attribute (omitted if the facet's property or attribute has no
     * unit).
     * The value set may be {@code null}.
     */
    public Facets setUnit(String unit) {
      this.unit = unit;
      return this;
    }

    /**
     * Model definition for ProductsFacetsBuckets.
     */
    public static final class Buckets extends GenericJson {

      /**
       * Number of products matching the query that have a value for the facet's property or attribute
       * that matches the bucket.
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private Integer count;

      /**
       * Upper bound of the bucket (omitted for value buckets or if the range has no upper bound).
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private Object max;

      /**
       * Whether the upper bound of the bucket is exclusive (omitted for value buckets or if the range
       * has no upper bound).
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private Boolean maxExclusive;

      /**
       * Lower bound of the bucket (omitted for value buckets or if the range has no lower bound).
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private Object min;

      /**
       * Whether the lower bound of the bucket is exclusive (omitted for value buckets or if the range
       * has no lower bound).
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private Boolean minExclusive;

      /**
       * Value of the bucket (omitted for range buckets).
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private Object value;

      /**
       * Number of products matching the query that have a value for the facet's property or attribute
       * that matches the bucket.
       * The value returned may be {@code null}.
       */
      public Integer getCount() {
        return count;
      }

      /**
       * Number of products matching the query that have a value for the facet's property or attribute
       * that matches the bucket.
       * The value set may be {@code null}.
       */
      public Buckets setCount(Integer count) {
        this.count = count;
        return this;
      }

      /**
       * Upper bound of the bucket (omitted for value buckets or if the range has no upper bound).
       * The value returned may be {@code null}.
       */
      public Object getMax() {
        return max;
      }

      /**
       * Upper bound of the bucket (omitted for value buckets or if the range has no upper bound).
       * The value set may be {@code null}.
       */
      public Buckets setMax(Object max) {
        this.max = max;
        return this;
      }

      /**
       * Whether the upper bound of the bucket is exclusive (omitted for value buckets or if the range
       * has no upper bound).
       * The value returned may be {@code null}.
       */
      public Boolean getMaxExclusive() {
        return maxExclusive;
      }

      /**
       * Whether the upper bound of the bucket is exclusive (omitted for value buckets or if the range
       * has no upper bound).
       * The value set may be {@code null}.
       */
      public Buckets setMaxExclusive(Boolean maxExclusive) {
        this.maxExclusive = maxExclusive;
        return this;
      }

      /**
       * Lower bound of the bucket (omitted for value buckets or if the range has no lower bound).
       * The value returned may be {@code null}.
       */
      public Object getMin() {
        return min;
      }

      /**
       * Lower bound of the bucket (omitted for value buckets or if the range has no lower bound).
       * The value set may be {@code null}.
       */
      public Buckets setMin(Object min) {
        this.min = min;
        return this;
      }

      /**
       * Whether the lower bound of the bucket is exclusive (omitted for value buckets or if the range
       * has no lower bound).
       * The value returned may be {@code null}.
       */
      public Boolean getMinExclusive() {
        return minExclusive;
      }

      /**
       * Whether the lower bound of the bucket is exclusive (omitted for value buckets or if the range
       * has no lower bound).
       * The value set may be {@code null}.
       */
      public Buckets setMinExclusive(Boolean minExclusive) {
        this.minExclusive = minExclusive;
        return this;
      }

      /**
       * Value of the bucket (omitted for range buckets).
       * The value returned may be {@code null}.
       */
      public Object getValue() {
        return value;
      }

      /**
       * Value of the bucket (omitted for range buckets).
       * The value set may be {@code null}.
       */
      public Buckets setValue(Object value) {
        this.value = value;
        return this;
      }

    }
  }

  /**
   * Model definition for ProductsPromotions.
   */
  public static final class Promotions extends GenericJson {

    /**
     * List of custom fields of promotion.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<CustomFields> customFields;

    static {
      // hack to force ProGuard to consider CustomFields used, since otherwise it would be stripped out
      // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
      com.google.api.client.util.Data.nullOf(CustomFields.class);
    }

    /**
     * Custom HTML of promotion (omitted if type is not custom).
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String customHtml;

    /**
     * Description of promotion (omitted if type is not standard).
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String description;

    /**
     * Link to promotion (omitted if type is not standard).
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String destLink;

    /**
     * Link to promotion image (omitted if type is not standard).
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String imageLink;

    /**
     * Name of promotion (omitted if type is not standard).
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String name;

    /**
     * Product of promotion (omitted if type is not product).
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private ShoppingModelProductJsonV1 product;

    /**
     * Type of promotion (one of: standard, product, custom).
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String type;

    /**
     * List of custom fields of promotion.
     * The value returned may be {@code null}.
     */
    public java.util.List<CustomFields> getCustomFields() {
      return customFields;
    }

    /**
     * List of custom fields of promotion.
     * The value set may be {@code null}.
     */
    public Promotions setCustomFields(java.util.List<CustomFields> customFields) {
      this.customFields = customFields;
      return this;
    }

    /**
     * Custom HTML of promotion (omitted if type is not custom).
     * The value returned may be {@code null}.
     */
    public String getCustomHtml() {
      return customHtml;
    }

    /**
     * Custom HTML of promotion (omitted if type is not custom).
     * The value set may be {@code null}.
     */
    public Promotions setCustomHtml(String customHtml) {
      this.customHtml = customHtml;
      return this;
    }

    /**
     * Description of promotion (omitted if type is not standard).
     * The value returned may be {@code null}.
     */
    public String getDescription() {
      return description;
    }

    /**
     * Description of promotion (omitted if type is not standard).
     * The value set may be {@code null}.
     */
    public Promotions setDescription(String description) {
      this.description = description;
      return this;
    }

    /**
     * Link to promotion (omitted if type is not standard).
     * The value returned may be {@code null}.
     */
    public String getDestLink() {
      return destLink;
    }

    /**
     * Link to promotion (omitted if type is not standard).
     * The value set may be {@code null}.
     */
    public Promotions setDestLink(String destLink) {
      this.destLink = destLink;
      return this;
    }

    /**
     * Link to promotion image (omitted if type is not standard).
     * The value returned may be {@code null}.
     */
    public String getImageLink() {
      return imageLink;
    }

    /**
     * Link to promotion image (omitted if type is not standard).
     * The value set may be {@code null}.
     */
    public Promotions setImageLink(String imageLink) {
      this.imageLink = imageLink;
      return this;
    }

    /**
     * Name of promotion (omitted if type is not standard).
     * The value returned may be {@code null}.
     */
    public String getName() {
      return name;
    }

    /**
     * Name of promotion (omitted if type is not standard).
     * The value set may be {@code null}.
     */
    public Promotions setName(String name) {
      this.name = name;
      return this;
    }

    /**
     * Product of promotion (omitted if type is not product).
     * The value returned may be {@code null}.
     */
    public ShoppingModelProductJsonV1 getProduct() {
      return product;
    }

    /**
     * Product of promotion (omitted if type is not product).
     * The value set may be {@code null}.
     */
    public Promotions setProduct(ShoppingModelProductJsonV1 product) {
      this.product = product;
      return this;
    }

    /**
     * Type of promotion (one of: standard, product, custom).
     * The value returned may be {@code null}.
     */
    public String getType() {
      return type;
    }

    /**
     * Type of promotion (one of: standard, product, custom).
     * The value set may be {@code null}.
     */
    public Promotions setType(String type) {
      this.type = type;
      return this;
    }

    /**
     * Model definition for ProductsPromotionsCustomFields.
     */
    public static final class CustomFields extends GenericJson {

      /**
       * Name of field.
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private String name;

      /**
       * Value of field.
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private String value;

      /**
       * Name of field.
       * The value returned may be {@code null}.
       */
      public String getName() {
        return name;
      }

      /**
       * Name of field.
       * The value set may be {@code null}.
       */
      public CustomFields setName(String name) {
        this.name = name;
        return this;
      }

      /**
       * Value of field.
       * The value returned may be {@code null}.
       */
      public String getValue() {
        return value;
      }

      /**
       * Value of field.
       * The value set may be {@code null}.
       */
      public CustomFields setValue(String value) {
        this.value = value;
        return this;
      }

    }
  }

  /**
   * Spelling.
   */
  public static final class Spelling extends GenericJson {

    /**
     * Suggestion for spelling.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String suggestion;

    /**
     * Suggestion for spelling.
     * The value returned may be {@code null}.
     */
    public String getSuggestion() {
      return suggestion;
    }

    /**
     * Suggestion for spelling.
     * The value set may be {@code null}.
     */
    public Spelling setSuggestion(String suggestion) {
      this.suggestion = suggestion;
      return this;
    }

  }

  /**
   * Model definition for ProductsStores.
   */
  public static final class Stores extends GenericJson {

    /**
     * Address of store.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String address;

    /**
     * Location of store.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String location;

    /**
     * Name of merchant.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String name;

    /**
     * Merchant-supplied store code.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String storeCode;

    /**
     * Id of store.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String storeId;

    /**
     * Name of store.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String storeName;

    /**
     * Telephone number of store.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String telephone;

    /**
     * Address of store.
     * The value returned may be {@code null}.
     */
    public String getAddress() {
      return address;
    }

    /**
     * Address of store.
     * The value set may be {@code null}.
     */
    public Stores setAddress(String address) {
      this.address = address;
      return this;
    }

    /**
     * Location of store.
     * The value returned may be {@code null}.
     */
    public String getLocation() {
      return location;
    }

    /**
     * Location of store.
     * The value set may be {@code null}.
     */
    public Stores setLocation(String location) {
      this.location = location;
      return this;
    }

    /**
     * Name of merchant.
     * The value returned may be {@code null}.
     */
    public String getName() {
      return name;
    }

    /**
     * Name of merchant.
     * The value set may be {@code null}.
     */
    public Stores setName(String name) {
      this.name = name;
      return this;
    }

    /**
     * Merchant-supplied store code.
     * The value returned may be {@code null}.
     */
    public String getStoreCode() {
      return storeCode;
    }

    /**
     * Merchant-supplied store code.
     * The value set may be {@code null}.
     */
    public Stores setStoreCode(String storeCode) {
      this.storeCode = storeCode;
      return this;
    }

    /**
     * Id of store.
     * The value returned may be {@code null}.
     */
    public String getStoreId() {
      return storeId;
    }

    /**
     * Id of store.
     * The value set may be {@code null}.
     */
    public Stores setStoreId(String storeId) {
      this.storeId = storeId;
      return this;
    }

    /**
     * Name of store.
     * The value returned may be {@code null}.
     */
    public String getStoreName() {
      return storeName;
    }

    /**
     * Name of store.
     * The value set may be {@code null}.
     */
    public Stores setStoreName(String storeName) {
      this.storeName = storeName;
      return this;
    }

    /**
     * Telephone number of store.
     * The value returned may be {@code null}.
     */
    public String getTelephone() {
      return telephone;
    }

    /**
     * Telephone number of store.
     * The value set may be {@code null}.
     */
    public Stores setTelephone(String telephone) {
      this.telephone = telephone;
      return this;
    }

  }

}
