/*
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
/*
 * Warning! This file is generated. Modify at your own risk.
 */

package com.google.api.services.shopping.model;

import com.google.api.client.json.GenericJson;
import com.google.api.client.json.JsonString;
import com.google.api.client.util.DateTime;
import com.google.common.primitives.UnsignedLong;

/**
 * Model definition for ShoppingModelProductJsonV1.
 *
 * <p> This is the Java data model class that specifies how to parse/serialize into the JSON that is
 * transmitted over HTTP when working with the Search API For Shopping. For a detailed explanation
 * see:
 * <a href="http://code.google.com/p/google-api-java-client/wiki/Json">http://code.google.com/p/google-api-java-client/wiki/Json</a>
 * </p>
 *
 * <p>
 * Upgrade warning: starting with version 1.12 {@code getResponseHeaders()} is removed, instead use
 * {@link com.google.api.client.http.json.JsonHttpRequest#getLastResponseHeaders()}
 * </p>
 *
 * @author Google, Inc.
 */
@SuppressWarnings("javadoc")
public final class ShoppingModelProductJsonV1 extends GenericJson {

  /**
   * Attributes of product (available only with a cx source).
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.List<Attributes> attributes;

  static {
    // hack to force ProGuard to consider Attributes used, since otherwise it would be stripped out
    // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
    com.google.api.client.util.Data.nullOf(Attributes.class);
  }

  /**
   * Author of product.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Author author;

  /**
   * Brand of product.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String brand;

  /**
   * Categories of product according to the selected taxonomy, omitted if no taxonomy is selected.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.List<String> categories;

  /**
   * Condition of product (one of: new, refurbished, used).
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String condition;

  /**
   * ISO 3166 code of target country of product.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String country;

  /**
   * RFC 3339 formatted creation time and date of product.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private DateTime creationTime;

  /**
   * Description of product.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String description;

  /**
   * Google id of product.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key @JsonString
  private UnsignedLong googleId;

  /**
   * The first GTIN of the product. Deprecated in favor of "gtins".
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String gtin;

  /**
   * List of all the product's GTINs (in GTIN-14 format).
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.List<String> gtins;

  /**
   * Images of product.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.List<Images> images;

  static {
    // hack to force ProGuard to consider Images used, since otherwise it would be stripped out
    // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
    com.google.api.client.util.Data.nullOf(Images.class);
  }

  /**
   * Google Internal.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.List<String> internal1;

  /**
   * Google Internal.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.List<String> internal10;

  /**
   * Google Internal.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String internal12;

  /**
   * Google Internal.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Double internal13;

  /**
   * Google Internal.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Double internal14;

  /**
   * Google Internal.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Double internal15;

  /**
   * Google Internal. Attribute names are deliberately vague.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Internal16 internal16;

  /**
   * Google Internal.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String internal3;

  /**
   * Google Internal.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.List<Internal4> internal4;

  static {
    // hack to force ProGuard to consider Internal4 used, since otherwise it would be stripped out
    // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
    com.google.api.client.util.Data.nullOf(Internal4.class);
  }

  /**
   * Google Internal.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String internal6;

  /**
   * Google Internal.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Boolean internal7;

  /**
   * Google Internal.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.List<String> internal8;

  /**
   * Inventories of product.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.List<Inventories> inventories;

  static {
    // hack to force ProGuard to consider Inventories used, since otherwise it would be stripped out
    // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
    com.google.api.client.util.Data.nullOf(Inventories.class);
  }

  /**
   * BCP 47 language tag of language of product.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String language;

  /**
   * Link to product.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String link;

  /**
   * RFC 3339 formatted modification time and date of product.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private DateTime modificationTime;

  /**
   * List of all the product's MPNs.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.List<String> mpns;

  /**
   * Code to add to the page to render the +1 content.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String plusOne;

  /**
   * Merchant-provided id of product (available only with a cx source).
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String providedId;

  /**
   * Whether this product matched the user query. Only set for the variant offers (if any) attached
   * to a product offer.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Boolean queryMatched;

  /**
   * Google Internal
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Float score;

  /**
   * Title of product.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String title;

  /**
   * The number of variant offers returned that matched the query.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Integer totalMatchingVariants;

  /**
   * A list of variant offers associated with this product.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private java.util.List<Variants> variants;

  static {
    // hack to force ProGuard to consider Variants used, since otherwise it would be stripped out
    // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
    com.google.api.client.util.Data.nullOf(Variants.class);
  }

  /**
   * Attributes of product (available only with a cx source).
   * The value returned may be {@code null}.
   */
  public java.util.List<Attributes> getAttributes() {
    return attributes;
  }

  /**
   * Attributes of product (available only with a cx source).
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setAttributes(java.util.List<Attributes> attributes) {
    this.attributes = attributes;
    return this;
  }

  /**
   * Author of product.
   * The value returned may be {@code null}.
   */
  public Author getAuthor() {
    return author;
  }

  /**
   * Author of product.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setAuthor(Author author) {
    this.author = author;
    return this;
  }

  /**
   * Brand of product.
   * The value returned may be {@code null}.
   */
  public String getBrand() {
    return brand;
  }

  /**
   * Brand of product.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setBrand(String brand) {
    this.brand = brand;
    return this;
  }

  /**
   * Categories of product according to the selected taxonomy, omitted if no taxonomy is selected.
   * The value returned may be {@code null}.
   */
  public java.util.List<String> getCategories() {
    return categories;
  }

  /**
   * Categories of product according to the selected taxonomy, omitted if no taxonomy is selected.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setCategories(java.util.List<String> categories) {
    this.categories = categories;
    return this;
  }

  /**
   * Condition of product (one of: new, refurbished, used).
   * The value returned may be {@code null}.
   */
  public String getCondition() {
    return condition;
  }

  /**
   * Condition of product (one of: new, refurbished, used).
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setCondition(String condition) {
    this.condition = condition;
    return this;
  }

  /**
   * ISO 3166 code of target country of product.
   * The value returned may be {@code null}.
   */
  public String getCountry() {
    return country;
  }

  /**
   * ISO 3166 code of target country of product.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setCountry(String country) {
    this.country = country;
    return this;
  }

  /**
   * RFC 3339 formatted creation time and date of product.
   * The value returned may be {@code null}.
   */
  public DateTime getCreationTime() {
    return creationTime;
  }

  /**
   * RFC 3339 formatted creation time and date of product.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setCreationTime(DateTime creationTime) {
    this.creationTime = creationTime;
    return this;
  }

  /**
   * Description of product.
   * The value returned may be {@code null}.
   */
  public String getDescription() {
    return description;
  }

  /**
   * Description of product.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setDescription(String description) {
    this.description = description;
    return this;
  }

  /**
   * Google id of product.
   * The value returned may be {@code null}.
   */
  public UnsignedLong getGoogleId() {
    return googleId;
  }

  /**
   * Google id of product.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setGoogleId(UnsignedLong googleId) {
    this.googleId = googleId;
    return this;
  }

  /**
   * The first GTIN of the product. Deprecated in favor of "gtins".
   * The value returned may be {@code null}.
   */
  public String getGtin() {
    return gtin;
  }

  /**
   * The first GTIN of the product. Deprecated in favor of "gtins".
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setGtin(String gtin) {
    this.gtin = gtin;
    return this;
  }

  /**
   * List of all the product's GTINs (in GTIN-14 format).
   * The value returned may be {@code null}.
   */
  public java.util.List<String> getGtins() {
    return gtins;
  }

  /**
   * List of all the product's GTINs (in GTIN-14 format).
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setGtins(java.util.List<String> gtins) {
    this.gtins = gtins;
    return this;
  }

  /**
   * Images of product.
   * The value returned may be {@code null}.
   */
  public java.util.List<Images> getImages() {
    return images;
  }

  /**
   * Images of product.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setImages(java.util.List<Images> images) {
    this.images = images;
    return this;
  }

  /**
   * Google Internal.
   * The value returned may be {@code null}.
   */
  public java.util.List<String> getInternal1() {
    return internal1;
  }

  /**
   * Google Internal.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setInternal1(java.util.List<String> internal1) {
    this.internal1 = internal1;
    return this;
  }

  /**
   * Google Internal.
   * The value returned may be {@code null}.
   */
  public java.util.List<String> getInternal10() {
    return internal10;
  }

  /**
   * Google Internal.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setInternal10(java.util.List<String> internal10) {
    this.internal10 = internal10;
    return this;
  }

  /**
   * Google Internal.
   * The value returned may be {@code null}.
   */
  public String getInternal12() {
    return internal12;
  }

  /**
   * Google Internal.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setInternal12(String internal12) {
    this.internal12 = internal12;
    return this;
  }

  /**
   * Google Internal.
   * The value returned may be {@code null}.
   */
  public Double getInternal13() {
    return internal13;
  }

  /**
   * Google Internal.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setInternal13(Double internal13) {
    this.internal13 = internal13;
    return this;
  }

  /**
   * Google Internal.
   * The value returned may be {@code null}.
   */
  public Double getInternal14() {
    return internal14;
  }

  /**
   * Google Internal.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setInternal14(Double internal14) {
    this.internal14 = internal14;
    return this;
  }

  /**
   * Google Internal.
   * The value returned may be {@code null}.
   */
  public Double getInternal15() {
    return internal15;
  }

  /**
   * Google Internal.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setInternal15(Double internal15) {
    this.internal15 = internal15;
    return this;
  }

  /**
   * Google Internal. Attribute names are deliberately vague.
   * The value returned may be {@code null}.
   */
  public Internal16 getInternal16() {
    return internal16;
  }

  /**
   * Google Internal. Attribute names are deliberately vague.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setInternal16(Internal16 internal16) {
    this.internal16 = internal16;
    return this;
  }

  /**
   * Google Internal.
   * The value returned may be {@code null}.
   */
  public String getInternal3() {
    return internal3;
  }

  /**
   * Google Internal.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setInternal3(String internal3) {
    this.internal3 = internal3;
    return this;
  }

  /**
   * Google Internal.
   * The value returned may be {@code null}.
   */
  public java.util.List<Internal4> getInternal4() {
    return internal4;
  }

  /**
   * Google Internal.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setInternal4(java.util.List<Internal4> internal4) {
    this.internal4 = internal4;
    return this;
  }

  /**
   * Google Internal.
   * The value returned may be {@code null}.
   */
  public String getInternal6() {
    return internal6;
  }

  /**
   * Google Internal.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setInternal6(String internal6) {
    this.internal6 = internal6;
    return this;
  }

  /**
   * Google Internal.
   * The value returned may be {@code null}.
   */
  public Boolean getInternal7() {
    return internal7;
  }

  /**
   * Google Internal.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setInternal7(Boolean internal7) {
    this.internal7 = internal7;
    return this;
  }

  /**
   * Google Internal.
   * The value returned may be {@code null}.
   */
  public java.util.List<String> getInternal8() {
    return internal8;
  }

  /**
   * Google Internal.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setInternal8(java.util.List<String> internal8) {
    this.internal8 = internal8;
    return this;
  }

  /**
   * Inventories of product.
   * The value returned may be {@code null}.
   */
  public java.util.List<Inventories> getInventories() {
    return inventories;
  }

  /**
   * Inventories of product.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setInventories(java.util.List<Inventories> inventories) {
    this.inventories = inventories;
    return this;
  }

  /**
   * BCP 47 language tag of language of product.
   * The value returned may be {@code null}.
   */
  public String getLanguage() {
    return language;
  }

  /**
   * BCP 47 language tag of language of product.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setLanguage(String language) {
    this.language = language;
    return this;
  }

  /**
   * Link to product.
   * The value returned may be {@code null}.
   */
  public String getLink() {
    return link;
  }

  /**
   * Link to product.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setLink(String link) {
    this.link = link;
    return this;
  }

  /**
   * RFC 3339 formatted modification time and date of product.
   * The value returned may be {@code null}.
   */
  public DateTime getModificationTime() {
    return modificationTime;
  }

  /**
   * RFC 3339 formatted modification time and date of product.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setModificationTime(DateTime modificationTime) {
    this.modificationTime = modificationTime;
    return this;
  }

  /**
   * List of all the product's MPNs.
   * The value returned may be {@code null}.
   */
  public java.util.List<String> getMpns() {
    return mpns;
  }

  /**
   * List of all the product's MPNs.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setMpns(java.util.List<String> mpns) {
    this.mpns = mpns;
    return this;
  }

  /**
   * Code to add to the page to render the +1 content.
   * The value returned may be {@code null}.
   */
  public String getPlusOne() {
    return plusOne;
  }

  /**
   * Code to add to the page to render the +1 content.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setPlusOne(String plusOne) {
    this.plusOne = plusOne;
    return this;
  }

  /**
   * Merchant-provided id of product (available only with a cx source).
   * The value returned may be {@code null}.
   */
  public String getProvidedId() {
    return providedId;
  }

  /**
   * Merchant-provided id of product (available only with a cx source).
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setProvidedId(String providedId) {
    this.providedId = providedId;
    return this;
  }

  /**
   * Whether this product matched the user query. Only set for the variant offers (if any) attached
   * to a product offer.
   * The value returned may be {@code null}.
   */
  public Boolean getQueryMatched() {
    return queryMatched;
  }

  /**
   * Whether this product matched the user query. Only set for the variant offers (if any) attached
   * to a product offer.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setQueryMatched(Boolean queryMatched) {
    this.queryMatched = queryMatched;
    return this;
  }

  /**
   * Google Internal
   * The value returned may be {@code null}.
   */
  public Float getScore() {
    return score;
  }

  /**
   * Google Internal
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setScore(Float score) {
    this.score = score;
    return this;
  }

  /**
   * Title of product.
   * The value returned may be {@code null}.
   */
  public String getTitle() {
    return title;
  }

  /**
   * Title of product.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setTitle(String title) {
    this.title = title;
    return this;
  }

  /**
   * The number of variant offers returned that matched the query.
   * The value returned may be {@code null}.
   */
  public Integer getTotalMatchingVariants() {
    return totalMatchingVariants;
  }

  /**
   * The number of variant offers returned that matched the query.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setTotalMatchingVariants(Integer totalMatchingVariants) {
    this.totalMatchingVariants = totalMatchingVariants;
    return this;
  }

  /**
   * A list of variant offers associated with this product.
   * The value returned may be {@code null}.
   */
  public java.util.List<Variants> getVariants() {
    return variants;
  }

  /**
   * A list of variant offers associated with this product.
   * The value set may be {@code null}.
   */
  public ShoppingModelProductJsonV1 setVariants(java.util.List<Variants> variants) {
    this.variants = variants;
    return this;
  }

  /**
   * Model definition for ShoppingModelProductJsonV1Attributes.
   */
  public static final class Attributes extends GenericJson {

    /**
     * Display Name of prodct attribute.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String displayName;

    /**
     * Name of product attribute.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String name;

    /**
     * Type of product attribute (one of: text, bool, int, float, dateRange, url).
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String type;

    /**
     * Unit of product attribute.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String unit;

    /**
     * Value of product attribute.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private Object value;

    /**
     * Display Name of prodct attribute.
     * The value returned may be {@code null}.
     */
    public String getDisplayName() {
      return displayName;
    }

    /**
     * Display Name of prodct attribute.
     * The value set may be {@code null}.
     */
    public Attributes setDisplayName(String displayName) {
      this.displayName = displayName;
      return this;
    }

    /**
     * Name of product attribute.
     * The value returned may be {@code null}.
     */
    public String getName() {
      return name;
    }

    /**
     * Name of product attribute.
     * The value set may be {@code null}.
     */
    public Attributes setName(String name) {
      this.name = name;
      return this;
    }

    /**
     * Type of product attribute (one of: text, bool, int, float, dateRange, url).
     * The value returned may be {@code null}.
     */
    public String getType() {
      return type;
    }

    /**
     * Type of product attribute (one of: text, bool, int, float, dateRange, url).
     * The value set may be {@code null}.
     */
    public Attributes setType(String type) {
      this.type = type;
      return this;
    }

    /**
     * Unit of product attribute.
     * The value returned may be {@code null}.
     */
    public String getUnit() {
      return unit;
    }

    /**
     * Unit of product attribute.
     * The value set may be {@code null}.
     */
    public Attributes setUnit(String unit) {
      this.unit = unit;
      return this;
    }

    /**
     * Value of product attribute.
     * The value returned may be {@code null}.
     */
    public Object getValue() {
      return value;
    }

    /**
     * Value of product attribute.
     * The value set may be {@code null}.
     */
    public Attributes setValue(Object value) {
      this.value = value;
      return this;
    }

  }

  /**
   * Author of product.
   */
  public static final class Author extends GenericJson {

    /**
     * Account id of product author.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key @JsonString
    private UnsignedLong accountId;

    /**
     * Name of product author.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String name;

    /**
     * Account id of product author.
     * The value returned may be {@code null}.
     */
    public UnsignedLong getAccountId() {
      return accountId;
    }

    /**
     * Account id of product author.
     * The value set may be {@code null}.
     */
    public Author setAccountId(UnsignedLong accountId) {
      this.accountId = accountId;
      return this;
    }

    /**
     * Name of product author.
     * The value returned may be {@code null}.
     */
    public String getName() {
      return name;
    }

    /**
     * Name of product author.
     * The value set may be {@code null}.
     */
    public Author setName(String name) {
      this.name = name;
      return this;
    }

  }

  /**
   * Model definition for ShoppingModelProductJsonV1Images.
   */
  public static final class Images extends GenericJson {

    /**
     * Link to product image.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String link;

    /**
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String status;

    /**
     * Thumbnails of product image.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<Thumbnails> thumbnails;

    static {
      // hack to force ProGuard to consider Thumbnails used, since otherwise it would be stripped out
      // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
      com.google.api.client.util.Data.nullOf(Thumbnails.class);
    }

    /**
     * Link to product image.
     * The value returned may be {@code null}.
     */
    public String getLink() {
      return link;
    }

    /**
     * Link to product image.
     * The value set may be {@code null}.
     */
    public Images setLink(String link) {
      this.link = link;
      return this;
    }

    /**

     * The value returned may be {@code null}.
     */
    public String getStatus() {
      return status;
    }

    /**

     * The value set may be {@code null}.
     */
    public Images setStatus(String status) {
      this.status = status;
      return this;
    }

    /**
     * Thumbnails of product image.
     * The value returned may be {@code null}.
     */
    public java.util.List<Thumbnails> getThumbnails() {
      return thumbnails;
    }

    /**
     * Thumbnails of product image.
     * The value set may be {@code null}.
     */
    public Images setThumbnails(java.util.List<Thumbnails> thumbnails) {
      this.thumbnails = thumbnails;
      return this;
    }

    /**
     * Model definition for ShoppingModelProductJsonV1ImagesThumbnails.
     */
    public static final class Thumbnails extends GenericJson {

      /**
       * Content of thumbnail (only available for the first thumbnail of the top results if SAYT is
       * enabled).
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private String content;

      /**
       * Height of thumbnail (omitted if not specified in the request).
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private Integer height;

      /**
       * Link to thumbnail.
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private String link;

      /**
       * Width of thumbnail (omitted if not specified in the request).
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private Integer width;

      /**
       * Content of thumbnail (only available for the first thumbnail of the top results if SAYT is
       * enabled).
       * The value returned may be {@code null}.
       */
      public String getContent() {
        return content;
      }

      /**
       * Content of thumbnail (only available for the first thumbnail of the top results if SAYT is
       * enabled).
       * The value set may be {@code null}.
       */
      public Thumbnails setContent(String content) {
        this.content = content;
        return this;
      }

      /**
       * Height of thumbnail (omitted if not specified in the request).
       * The value returned may be {@code null}.
       */
      public Integer getHeight() {
        return height;
      }

      /**
       * Height of thumbnail (omitted if not specified in the request).
       * The value set may be {@code null}.
       */
      public Thumbnails setHeight(Integer height) {
        this.height = height;
        return this;
      }

      /**
       * Link to thumbnail.
       * The value returned may be {@code null}.
       */
      public String getLink() {
        return link;
      }

      /**
       * Link to thumbnail.
       * The value set may be {@code null}.
       */
      public Thumbnails setLink(String link) {
        this.link = link;
        return this;
      }

      /**
       * Width of thumbnail (omitted if not specified in the request).
       * The value returned may be {@code null}.
       */
      public Integer getWidth() {
        return width;
      }

      /**
       * Width of thumbnail (omitted if not specified in the request).
       * The value set may be {@code null}.
       */
      public Thumbnails setWidth(Integer width) {
        this.width = width;
        return this;
      }

    }
  }

  /**
   * Google Internal. Attribute names are deliberately vague.
   */
  public static final class Internal16 extends GenericJson {

    /**
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private Integer length;

    /**
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private Integer number;

    /**
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key @JsonString
    private Long size;

    /**

     * The value returned may be {@code null}.
     */
    public Integer getLength() {
      return length;
    }

    /**

     * The value set may be {@code null}.
     */
    public Internal16 setLength(Integer length) {
      this.length = length;
      return this;
    }

    /**

     * The value returned may be {@code null}.
     */
    public Integer getNumber() {
      return number;
    }

    /**

     * The value set may be {@code null}.
     */
    public Internal16 setNumber(Integer number) {
      this.number = number;
      return this;
    }

    /**

     * The value returned may be {@code null}.
     */
    public Long getSize() {
      return size;
    }

    /**

     * The value set may be {@code null}.
     */
    public Internal16 setSize(Long size) {
      this.size = size;
      return this;
    }

  }

  /**
   * Model definition for ShoppingModelProductJsonV1Internal4.
   */
  public static final class Internal4 extends GenericJson {

    /**
     * Google Internal.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private Double confidence;

    /**
     * Google Internal.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private Integer node;

    /**
     * Google Internal.
     * The value returned may be {@code null}.
     */
    public Double getConfidence() {
      return confidence;
    }

    /**
     * Google Internal.
     * The value set may be {@code null}.
     */
    public Internal4 setConfidence(Double confidence) {
      this.confidence = confidence;
      return this;
    }

    /**
     * Google Internal.
     * The value returned may be {@code null}.
     */
    public Integer getNode() {
      return node;
    }

    /**
     * Google Internal.
     * The value set may be {@code null}.
     */
    public Internal4 setNode(Integer node) {
      this.node = node;
      return this;
    }

  }

  /**
   * Model definition for ShoppingModelProductJsonV1Inventories.
   */
  public static final class Inventories extends GenericJson {

    /**
     * Availability of product inventory.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String availability;

    /**
     * Channel of product inventory (one of: online, local).
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String channel;

    /**
     * Currency of product inventory (an ISO 4217 alphabetic code).
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String currency;

    /**
     * Distance of product inventory.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private Float distance;

    /**
     * Distance unit of product inventory.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String distanceUnit;

    /**
     * Number of months for installment price.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private Integer installmentMonths;

    /**
     * Installment price of product inventory.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private Float installmentPrice;

    /**
     * Original price of product inventory. Only returned for products that are on sale.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private Float originalPrice;

    /**
     * Price of product inventory.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private Float price;

    /**
     * Sale end date.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private DateTime saleEndDate;

    /**
     * Sale price of product inventory.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private Float salePrice;

    /**
     * Sale start date.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private DateTime saleStartDate;

    /**
     * Shipping cost of product inventory.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private Float shipping;

    /**
     * Store ID of product inventory.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String storeId;

    /**
     * Tax of product inventory.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private Float tax;

    /**
     * Availability of product inventory.
     * The value returned may be {@code null}.
     */
    public String getAvailability() {
      return availability;
    }

    /**
     * Availability of product inventory.
     * The value set may be {@code null}.
     */
    public Inventories setAvailability(String availability) {
      this.availability = availability;
      return this;
    }

    /**
     * Channel of product inventory (one of: online, local).
     * The value returned may be {@code null}.
     */
    public String getChannel() {
      return channel;
    }

    /**
     * Channel of product inventory (one of: online, local).
     * The value set may be {@code null}.
     */
    public Inventories setChannel(String channel) {
      this.channel = channel;
      return this;
    }

    /**
     * Currency of product inventory (an ISO 4217 alphabetic code).
     * The value returned may be {@code null}.
     */
    public String getCurrency() {
      return currency;
    }

    /**
     * Currency of product inventory (an ISO 4217 alphabetic code).
     * The value set may be {@code null}.
     */
    public Inventories setCurrency(String currency) {
      this.currency = currency;
      return this;
    }

    /**
     * Distance of product inventory.
     * The value returned may be {@code null}.
     */
    public Float getDistance() {
      return distance;
    }

    /**
     * Distance of product inventory.
     * The value set may be {@code null}.
     */
    public Inventories setDistance(Float distance) {
      this.distance = distance;
      return this;
    }

    /**
     * Distance unit of product inventory.
     * The value returned may be {@code null}.
     */
    public String getDistanceUnit() {
      return distanceUnit;
    }

    /**
     * Distance unit of product inventory.
     * The value set may be {@code null}.
     */
    public Inventories setDistanceUnit(String distanceUnit) {
      this.distanceUnit = distanceUnit;
      return this;
    }

    /**
     * Number of months for installment price.
     * The value returned may be {@code null}.
     */
    public Integer getInstallmentMonths() {
      return installmentMonths;
    }

    /**
     * Number of months for installment price.
     * The value set may be {@code null}.
     */
    public Inventories setInstallmentMonths(Integer installmentMonths) {
      this.installmentMonths = installmentMonths;
      return this;
    }

    /**
     * Installment price of product inventory.
     * The value returned may be {@code null}.
     */
    public Float getInstallmentPrice() {
      return installmentPrice;
    }

    /**
     * Installment price of product inventory.
     * The value set may be {@code null}.
     */
    public Inventories setInstallmentPrice(Float installmentPrice) {
      this.installmentPrice = installmentPrice;
      return this;
    }

    /**
     * Original price of product inventory. Only returned for products that are on sale.
     * The value returned may be {@code null}.
     */
    public Float getOriginalPrice() {
      return originalPrice;
    }

    /**
     * Original price of product inventory. Only returned for products that are on sale.
     * The value set may be {@code null}.
     */
    public Inventories setOriginalPrice(Float originalPrice) {
      this.originalPrice = originalPrice;
      return this;
    }

    /**
     * Price of product inventory.
     * The value returned may be {@code null}.
     */
    public Float getPrice() {
      return price;
    }

    /**
     * Price of product inventory.
     * The value set may be {@code null}.
     */
    public Inventories setPrice(Float price) {
      this.price = price;
      return this;
    }

    /**
     * Sale end date.
     * The value returned may be {@code null}.
     */
    public DateTime getSaleEndDate() {
      return saleEndDate;
    }

    /**
     * Sale end date.
     * The value set may be {@code null}.
     */
    public Inventories setSaleEndDate(DateTime saleEndDate) {
      this.saleEndDate = saleEndDate;
      return this;
    }

    /**
     * Sale price of product inventory.
     * The value returned may be {@code null}.
     */
    public Float getSalePrice() {
      return salePrice;
    }

    /**
     * Sale price of product inventory.
     * The value set may be {@code null}.
     */
    public Inventories setSalePrice(Float salePrice) {
      this.salePrice = salePrice;
      return this;
    }

    /**
     * Sale start date.
     * The value returned may be {@code null}.
     */
    public DateTime getSaleStartDate() {
      return saleStartDate;
    }

    /**
     * Sale start date.
     * The value set may be {@code null}.
     */
    public Inventories setSaleStartDate(DateTime saleStartDate) {
      this.saleStartDate = saleStartDate;
      return this;
    }

    /**
     * Shipping cost of product inventory.
     * The value returned may be {@code null}.
     */
    public Float getShipping() {
      return shipping;
    }

    /**
     * Shipping cost of product inventory.
     * The value set may be {@code null}.
     */
    public Inventories setShipping(Float shipping) {
      this.shipping = shipping;
      return this;
    }

    /**
     * Store ID of product inventory.
     * The value returned may be {@code null}.
     */
    public String getStoreId() {
      return storeId;
    }

    /**
     * Store ID of product inventory.
     * The value set may be {@code null}.
     */
    public Inventories setStoreId(String storeId) {
      this.storeId = storeId;
      return this;
    }

    /**
     * Tax of product inventory.
     * The value returned may be {@code null}.
     */
    public Float getTax() {
      return tax;
    }

    /**
     * Tax of product inventory.
     * The value set may be {@code null}.
     */
    public Inventories setTax(Float tax) {
      this.tax = tax;
      return this;
    }

  }

  /**
   * Model definition for ShoppingModelProductJsonV1Variants.
   */
  public static final class Variants extends GenericJson {

    /**
     * The detailed offer data for a particular variant offer.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private ShoppingModelProductJsonV1 variant;

    /**
     * The detailed offer data for a particular variant offer.
     * The value returned may be {@code null}.
     */
    public ShoppingModelProductJsonV1 getVariant() {
      return variant;
    }

    /**
     * The detailed offer data for a particular variant offer.
     * The value set may be {@code null}.
     */
    public Variants setVariant(ShoppingModelProductJsonV1 variant) {
      this.variant = variant;
      return this;
    }

  }

}
